@@ -1,4 +1,5 @@
 /*
+ *  Copyright (C) 2015 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2009 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm, Trog, Török Edvin
@@ -40,10 +41,6 @@
 #include <unistd.h>
 #endif
 
-#include <openssl/ssl.h>
-#include <openssl/err.h>
-#include "libclamav/crypto.h"
-
 #include <fcntl.h>
 #ifdef C_SOLARIS
 #include <stdio_ext.h>
@@ -53,7 +50,7 @@
 #include "shared/output.h"
 #include "shared/optparser.h"
 
-#include "fan.h"
+#include "onaccess_fan.h"
 #include "server.h"
 #include "thrmgr.h"
 #include "session.h"
@@ -71,6 +68,7 @@ int reload = 0;
 time_t reloaded_time = 0;
 pthread_mutex_t reload_mutex = PTHREAD_MUTEX_INITIALIZER;
 int sighup = 0;
+extern pthread_mutex_t logg_mutex;
 static struct cl_stat dbstat;
 
 void *event_wake_recv = NULL;
@@ -299,6 +297,15 @@ static const char *get_cmd(struct fd_buf *buf, size_t off, size_t *len, char *te
     }
 }
 
+int statinidir_th(const char* dirname)
+{
+   if (!dbstat.entries) {
+      memset(&dbstat, 0, sizeof(dbstat));
+   }
+
+   return cl_statinidir(dirname, &dbstat);
+}
+
 struct acceptdata {
     struct fd_data fds;
     struct fd_data recv_fds;
@@ -877,6 +884,36 @@ int recvloop_th(int *socketds, unsigned nsockets, struct cl_engine *engine, unsi
     val = cl_engine_get_num(engine, CL_ENGINE_MAX_ICONSPE, NULL);
     logg("Limits: MaxIconsPE limit set to %llu.\n", val);
 
+    if((opt = optget(opts, "PCREMatchLimit"))->active) {
+        if((ret = cl_engine_set_num(engine, CL_ENGINE_PCRE_MATCH_LIMIT, opt->numarg))) {
+            logg("!cli_engine_set_num(PCREMatchLimit) failed: %s\n", cl_strerror(ret));
+            cl_engine_free(engine);
+            return 1;
+        }
+    }
+    val = cl_engine_get_num(engine, CL_ENGINE_PCRE_MATCH_LIMIT, NULL);
+    logg("Limits: PCREMatchLimit limit set to %llu.\n", val);
+
+    if((opt = optget(opts, "PCRERecMatchLimit"))->active) {
+        if((ret = cl_engine_set_num(engine, CL_ENGINE_PCRE_RECMATCH_LIMIT, opt->numarg))) {
+            logg("!cli_engine_set_num(PCRERecMatchLimit) failed: %s\n", cl_strerror(ret));
+            cl_engine_free(engine);
+            return 1;
+        }
+    }
+    val = cl_engine_get_num(engine, CL_ENGINE_PCRE_RECMATCH_LIMIT, NULL);
+    logg("Limits: PCRERecMatchLimit limit set to %llu.\n", val);
+
+    if((opt = optget(opts, "PCREMaxFileSize"))->active) {
+        if((ret = cl_engine_set_num(engine, CL_ENGINE_PCRE_MAX_FILESIZE, opt->numarg))) {
+            logg("!cli_engine_set_num(PCREMaxFileSize) failed: %s\n", cl_strerror(ret));
+            cl_engine_free(engine);
+            return 1;
+        }
+    }
+    val = cl_engine_get_num(engine, CL_ENGINE_PCRE_MAX_FILESIZE, NULL);
+    logg("Limits: PCREMaxFileSize limit set to %llu.\n", val);
+
     if(optget(opts, "ScanArchive")->enabled) {
 	logg("Archive support enabled.\n");
 	options |= CL_SCAN_ARCHIVE;
@@ -1027,7 +1064,6 @@ int recvloop_th(int *socketds, unsigned nsockets, struct cl_engine *engine, unsi
     } else {
 	logg("Self checking every %u seconds.\n", selfchk);
     }
-    memset(&dbstat, 0, sizeof(dbstat));
 
     /* save the PID */
     mainpid = getpid();
@@ -1037,7 +1073,7 @@ int recvloop_th(int *socketds, unsigned nsockets, struct cl_engine *engine, unsi
 	if((fd = fopen(opt->strarg, "w")) == NULL) {
 	    logg("!Can't save PID in file %s\n", opt->strarg);
 	} else {
-	    if (fprintf(fd, "%u", (unsigned int) mainpid)<0) {
+	    if (fprintf(fd, "%u\n", (unsigned int) mainpid)<0) {
 	    	logg("!Can't save PID in file %s\n", opt->strarg);
 	    }
 	    fclose(fd);
@@ -1105,6 +1141,7 @@ int recvloop_th(int *socketds, unsigned nsockets, struct cl_engine *engine, unsi
     acceptdata.max_queue = max_queue;
 
     if(optget(opts, "ScanOnAccess")->enabled)
+
 #if defined(FANOTIFY) || defined(CLAMAUTH)
     {
         do {
@@ -1114,7 +1151,7 @@ int recvloop_th(int *socketds, unsigned nsockets, struct cl_engine *engine, unsi
 	    tharg->opts = opts;
 	    tharg->engine = engine;
 	    tharg->options = options;
-	    if(!pthread_create(&fan_pid, &fan_attr, fan_th, tharg)) break;
+	    if(!pthread_create(&fan_pid, &fan_attr, onas_fan_th, tharg)) break;
 	    free(tharg);
 	    tharg=NULL;
 	} while(0);
@@ -1124,6 +1161,7 @@ int recvloop_th(int *socketds, unsigned nsockets, struct cl_engine *engine, unsi
 	logg("!On-access scan is not available\n");
 #endif
 
+
 #ifndef	_WIN32
     /* set up signal handling */
     sigfillset(&sigset);
@@ -1389,13 +1427,7 @@ int recvloop_th(int *socketds, unsigned nsockets, struct cl_engine *engine, unsi
 	pthread_mutex_lock(&reload_mutex);
 	if(reload) {
 	    pthread_mutex_unlock(&reload_mutex);
-#if defined(FANOTIFY) || defined(CLAMAUTH)
-	    if(optget(opts, "ScanOnAccess")->enabled && tharg) {
-		logg("Restarting on-access scan\n");
-		pthread_kill(fan_pid, SIGUSR1);
-		pthread_join(fan_pid, NULL);
-	    }
-#endif
+
 	    engine = reload_db(engine, dboptions, opts, FALSE, &ret);
 	    if(ret) {
 		logg("Terminating because of a fatal error.\n");
@@ -1408,10 +1440,10 @@ int recvloop_th(int *socketds, unsigned nsockets, struct cl_engine *engine, unsi
 	    reload = 0;
 	    time(&reloaded_time);
 	    pthread_mutex_unlock(&reload_mutex);
+
 #if defined(FANOTIFY) || defined(CLAMAUTH)
 	    if(optget(opts, "ScanOnAccess")->enabled && tharg) {
 		tharg->engine = engine;
-		pthread_create(&fan_pid, &fan_attr, fan_th, tharg);
 	    }
 #endif
 	    time(&start_time);
@@ -1438,8 +1470,11 @@ int recvloop_th(int *socketds, unsigned nsockets, struct cl_engine *engine, unsi
 #if defined(FANOTIFY) || defined(CLAMAUTH)
     if(optget(opts, "ScanOnAccess")->enabled && tharg) {
 	logg("Stopping on-access scan\n");
+	pthread_mutex_lock(&logg_mutex);
 	pthread_kill(fan_pid, SIGUSR1);
+	pthread_mutex_unlock(&logg_mutex);
 	pthread_join(fan_pid, NULL);
+    free(tharg);
     }
 #endif
     if(engine) {
@@ -1475,4 +1510,4 @@ int recvloop_th(int *socketds, unsigned nsockets, struct cl_engine *engine, unsi
     logg("--- Stopped at %s", cli_ctime(&current_time, timestr, sizeof(timestr)));
 
     return ret;
-}
+} 
