@@ -1,4 +1,5 @@
 /*
+ *  Copyright (C) 2015 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2009 Sourcefire, Inc.
  *
  *  Authors: Trog, Török Edvin
@@ -29,12 +30,9 @@
 #include <errno.h>
 #include <string.h>
 
-#include <openssl/ssl.h>
-#include <openssl/err.h>
-#include "libclamav/crypto.h"
-
 #include "shared/output.h"
 
+#include "libclamav/clamav.h"
 #include "thrmgr.h"
 #include "others.h"
 #include "mpool.h"
