@@ -60,10 +60,6 @@
 #include <errno.h>
 #include <zlib.h>
 
-#include <openssl/ssl.h>
-#include <openssl/err.h>
-#include "libclamav/crypto.h"
-
 #include "target.h"
 
 #include "freshclamcodes.h"
@@ -94,29 +90,6 @@ char g_label[33];
 	if(chdir(x) == -1)			\
 	    logg("!Can't chdir to %s\n", x);
 
-#ifndef HAVE_GETADDRINFO
-static const char *
-ghbn_err (int err)              /* hstrerror() */
-{
-    switch (err)
-    {
-    case HOST_NOT_FOUND:
-        return "Host not found";
-
-    case NO_DATA:
-        return "No IP address";
-
-    case NO_RECOVERY:
-        return "Unrecoverable DNS error";
-
-    case TRY_AGAIN:
-        return "Temporary DNS error";
-
-    default:
-        return "Unknown error";
-    }
-}
-#endif
 
 static int
 textrecordfield (const char * dbname)
@@ -159,7 +132,6 @@ getclientsock (const char *localip, int prot)
 
     if (localip)
     {
-#ifdef HAVE_GETADDRINFO
         struct addrinfo *res;
         int ret;
 
@@ -197,53 +169,17 @@ getclientsock (const char *localip, int prot)
             freeaddrinfo (res);
         }
 
-#else /* IPv4 */
-        struct hostent *he;
-
-        if (!(he = gethostbyname (localip)))
-        {
-            logg ("!Could not resolve local ip address '%s': %s\n", localip,
-                  ghbn_err (h_errno));
-            logg ("^Using standard local ip address and port for fetching.\n");
-        }
-        else
-        {
-            struct sockaddr_in client;
-            unsigned char *ia;
-            char ipaddr[16];
-
-            memset ((char *) &client, 0, sizeof (client));
-            client.sin_family = AF_INET;
-            client.sin_addr = *(struct in_addr *) he->h_addr_list[0];
-            if (bind
-                (socketfd, (struct sockaddr *) &client,
-                 (socklen_t)sizeof (struct sockaddr_in)) != 0)
-            {
-                logg ("!Could not bind to local ip address '%s': %s\n",
-                      localip, strerror (errno));
-                logg ("^Using default client ip.\n");
-            }
-            else
-            {
-                ia = (unsigned char *) he->h_addr_list[0];
-                sprintf (ipaddr, "%u.%u.%u.%u", ia[0], ia[1], ia[2], ia[3]);
-                logg ("*Using ip '%s' for fetching.\n", ipaddr);
-            }
-        }
-#endif
     }
 
     return socketfd;
 }
 
-#ifdef HAVE_GETADDRINFO
 static int
 qcompare (const void *a, const void *b)
 {
     return (*(const struct addrinfo **) a)->ai_flags -
         (*(const struct addrinfo **) b)->ai_flags;
 }
-#endif
 
 static int
 wwwconnect (const char *server, const char *proxy, int pport, char *ip,
@@ -252,18 +188,12 @@ wwwconnect (const char *server, const char *proxy, int pport, char *ip,
 {
     int socketfd, port, ret;
     unsigned int ips = 0, ignored = 0, i;
-#ifdef HAVE_GETADDRINFO
     struct addrinfo hints, *res = NULL, *rp, *loadbal_rp = NULL, *addrs[128];
     char port_s[6], loadbal_ipaddr[46];
     uint32_t loadbal = 1, minsucc = 0xffffffff, minfail =
         0xffffffff, addrnum = 0;
     int ipv4start = -1, ipv4end = -1;
     struct mirdat_ip *md;
-#else
-    struct sockaddr_in name;
-    struct hostent *host;
-    unsigned char *ia;
-#endif
     char ipaddr[46];
     const char *hostpt;
 
@@ -294,7 +224,6 @@ wwwconnect (const char *server, const char *proxy, int pport, char *ip,
         port = 80;
     }
 
-#ifdef HAVE_GETADDRINFO
     memset (&hints, 0, sizeof (hints));
 #ifdef SUPPORT_IPv6
     hints.ai_family = AF_UNSPEC;
@@ -454,7 +383,8 @@ wwwconnect (const char *server, const char *proxy, int pport, char *ip,
             }
             else
                 i++;
-            mirman_update (addr, rp->ai_family, mdat, 2);
+            if (mdat)
+                mirman_update (addr, rp->ai_family, mdat, 2);
             continue;
         }
         else
@@ -474,79 +404,6 @@ wwwconnect (const char *server, const char *proxy, int pport, char *ip,
     }
     freeaddrinfo (res);
 
-#else /* IPv4 */
-
-    if ((host = gethostbyname (hostpt)) == NULL)
-    {
-        logg ("%cCan't get information about %s: %s\n", logerr ? '!' : '^',
-              hostpt, ghbn_err (h_errno));
-        return -1;
-    }
-
-    for (i = 0; host->h_addr_list[i] != 0; i++)
-    {
-        /* this dirty hack comes from pink - Nosuid TCP/IP ping 1.6 */
-        ia = (unsigned char *) host->h_addr_list[i];
-        sprintf (ipaddr, "%u.%u.%u.%u", ia[0], ia[1], ia[2], ia[3]);
-
-        ips++;
-        if (mdat
-            && (ret =
-                mirman_check (&((struct in_addr *) ia)->s_addr, AF_INET, mdat,
-                              NULL)))
-        {
-            if (ret == 1)
-                logg ("*Ignoring mirror %s (due to previous errors)\n",
-                      ipaddr);
-            else
-                logg ("*Ignoring mirror %s (has connected too many times with an outdated version)\n", ipaddr);
-            ignored++;
-            continue;
-        }
-
-        if (ip)
-            strcpy (ip, ipaddr);
-
-        if (i > 0)
-            logg ("Trying host %s (%s)...\n", hostpt, ipaddr);
-
-        memset ((char *) &name, 0, sizeof (name));
-        name.sin_family = AF_INET;
-        name.sin_addr = *((struct in_addr *) host->h_addr_list[i]);
-        name.sin_port = htons (port);
-
-        socketfd = getclientsock (localip, AF_INET);
-        if (socketfd < 0)
-            return -1;
-
-#ifdef SO_ERROR
-        if (wait_connect
-            (socketfd, (struct sockaddr *) &name, sizeof (struct sockaddr_in),
-             ctimeout) == -1)
-        {
-#else
-        if (connect
-            (socketfd, (struct sockaddr *) &name,
-             sizeof (struct sockaddr_in)) == -1)
-        {
-#endif
-            logg ("Can't connect to port %d of host %s (IP: %s)\n", port,
-                  hostpt, ipaddr);
-            closesocket (socketfd);
-            continue;
-        }
-        else
-        {
-            if (mdat)
-            {
-                mdat->currip[0] = ((struct in_addr *) ia)->s_addr;
-                mdat->af = AF_INET;
-            }
-            return socketfd;
-        }
-    }
-#endif
-
     if (mdat && can_whitelist && ips && (ips == ignored))
         mirman_whitelist (mdat, 1);
 
@@ -885,6 +742,10 @@ getfile_mirman (const char *srcfile, const char *destfile,
     char *remotename = NULL, *authorization = NULL, *headerline;
     const char *rotation = "|/-\\", *fname;
 
+    UNUSEDPARAM(localip);
+    UNUSEDPARAM(port);
+    UNUSEDPARAM(ctimeout);
+    UNUSEDPARAM(can_whitelist);
 
     if (proxy)
     {
@@ -1068,7 +929,11 @@ getfile_mirman (const char *srcfile, const char *destfile,
         if (totalsize > 0)
             percentage = (int) (100 * (float) totaldownloaded / totalsize);
 
+#ifdef HAVE_UNISTD_H
+        if (!mprintf_quiet && isatty(fileno(stdout)))
+#else
         if (!mprintf_quiet)
+#endif
         {
             if (totalsize > 0)
             {
@@ -1118,6 +983,8 @@ getfile (const char *srcfile, const char *destfile, const char *hostname,
     int ret, sd;
     char ipaddr[46];
 
+    UNUSEDPARAM(opts);
+
     memset (ipaddr, 0, sizeof (ipaddr));
     if (ip && ip[0])            /* use ip to connect */
         sd = wwwconnect (ip, proxy, port, ipaddr, localip, ctimeout, mdat,
@@ -1207,13 +1074,21 @@ chdir_tmp (const char *dbname, const char *tmpdir)
 {
     char cvdfile[32];
 
-
     if (access (tmpdir, R_OK | W_OK) == -1)
     {
-        sprintf (cvdfile, "%s.cvd", dbname);
+        int ret;
+        ret = snprintf (cvdfile, sizeof(cvdfile), "%s.cvd", dbname);
+        if (ret >= sizeof(cvdfile) || ret == -1) {
+            logg ("!chdir_tmp: dbname parameter value too long to create cvd file name: %s\n", dbname);
+            return -1;
+        }
         if (access (cvdfile, R_OK) == -1)
         {
-            sprintf (cvdfile, "%s.cld", dbname);
+            ret = snprintf (cvdfile, sizeof(cvdfile), "%s.cld", dbname);
+            if (ret >= sizeof(cvdfile) || ret == -1) {
+                logg ("!chdir_tmp: dbname parameter value too long to create cld file name: %s\n", dbname);
+                return -1;
+            }
             if (access (cvdfile, R_OK) == -1)
             {
                 logg ("!chdir_tmp: Can't access local %s database\n", dbname);
@@ -2050,7 +1925,7 @@ updatedb (const char *dbname, const char *hostname, char *ip, int *signo,
             logg ("^Current functionality level = %d, recommended = %d\n",
                   flevel, current->fl);
             logg ("Please check if ClamAV tools are linked against the proper version of libclamav\n");
-            logg ("DON'T PANIC! Read http://www.clamav.net/support/faq\n");
+            logg ("DON'T PANIC! Read http://www.clamav.net/doc/install.html\n");
         }
 
         *signo += current->sigs;
@@ -2551,9 +2426,7 @@ downloadmanager (const struct optstruct *opts, const char *hostname,
 
     time (&currtime);
     logg ("ClamAV update process started at %s", ctime (&currtime));
-#ifdef HAVE_GETADDRINFO
     logg ("*Using IPv6 aware code\n");
-#endif
 
 #ifdef HAVE_RESOLV_H
     dnsdbinfo = optget (opts, "DNSDatabaseInfo")->strarg;
@@ -2851,7 +2724,7 @@ downloadmanager (const struct optstruct *opts, const char *hostname,
 
     cli_rmdirs (updtmpdir);
 
-    if (checkdbdir () < 0)
+    if (updated && checkdbdir () < 0)
     {
         if (newver)
             free (newver);
