@@ -8,10 +8,6 @@
 #include <Windows.h>
 #endif
 
-#include <openssl/ssl.h>
-#include <openssl/err.h>
-#include "libclamav/crypto.h"
-
 #include "7z.h"
 #include "7zCrc.h"
 #include "CpuArch.h"
@@ -23,6 +19,9 @@ Byte k7zSignature[k7zSignatureSize] = {'7', 'z', 0xBC, 0xAF, 0x27, 0x1C};
 #define NUM_FOLDER_CODERS_MAX 32
 #define NUM_CODER_STREAMS_MAX 32
 
+void SzFolder_Free(CSzFolder *p, ISzAlloc *alloc);
+int SzFolder_FindBindPairForOutStream(CSzFolder *p, UInt32 outStreamIndex);
+
 void SzCoderInfo_Init(CSzCoderInfo *p)
 {
   Buf_Init(&p->Props);
