@@ -1,6 +1,7 @@
 /*
  *  JIT compile ClamAV bytecode.
  *
+ *  Copyright (C) 2015 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2009-2010 Sourcefire, Inc.
  *
  *  Authors: Török Edvin
@@ -24,10 +25,16 @@
 #ifndef _WIN32
 #include <sys/time.h>
 #endif
+#include <cstdlib>
+#include <csetjmp>
+#include <new>
+#include <cerrno>
+#include <string>
 
 #include "ClamBCModule.h"
 #include "ClamBCDiagnostics.h"
-#include "llvm/Analysis/DebugInfo.h"
+#include "llvm30_compat.h"
+
 #include "llvm/ADT/DenseMap.h"
 #include "llvm/ADT/BitVector.h"
 #include "llvm/ADT/PostOrderIterator.h"
@@ -38,17 +45,18 @@
 #include "llvm/ADT/SmallVector.h"
 #include "llvm/Analysis/LoopInfo.h"
 #include "llvm/Analysis/ScalarEvolution.h"
+#if LLVM_VERSION < 35
 #include "llvm/Analysis/Verifier.h"
 #include "llvm/AutoUpgrade.h"
-#include "llvm/CallingConv.h"
-#include "llvm/DerivedTypes.h"
-#include "llvm/Function.h"
+#include "llvm/Support/TargetFolder.h"
+#else
+#include "llvm/IR/Verifier.h"
+#include "llvm/IR/AutoUpgrade.h"
+#include "llvm/Analysis/TargetFolder.h"
+#endif
 #include "llvm/ExecutionEngine/ExecutionEngine.h"
 #include "llvm/ExecutionEngine/JIT.h"
 #include "llvm/ExecutionEngine/JITEventListener.h"
-#include "llvm/LLVMContext.h"
-#include "llvm/Intrinsics.h"
-#include "llvm/Module.h"
 #include "llvm/PassManager.h"
 #include "llvm/Support/Compiler.h"
 #include "llvm/Support/Debug.h"
@@ -58,10 +66,18 @@
 #include "llvm/Support/MemoryBuffer.h"
 #include "llvm/Support/raw_ostream.h"
 #include "llvm/Support/SourceMgr.h"
-#include "llvm/Support/IRBuilder.h"
 #include "llvm/Support/PrettyStackTrace.h"
 
-#ifdef LLVM29
+#if LLVM_VERSION < 29
+#include "llvm/System/DataTypes.h"
+#include "llvm/System/Host.h"
+#include "llvm/System/Memory.h"
+#include "llvm/System/Mutex.h"
+#include "llvm/System/Signals.h"
+#include "llvm/System/Threading.h"
+#include "llvm/System/ThreadLocal.h"
+#else
+#include "llvm/PassRegistry.h"
 #include "llvm/Support/DataTypes.h"
 #include "llvm/Support/FileSystem.h"
 #include "llvm/Support/Host.h"
@@ -70,16 +86,12 @@
 #include "llvm/Support/Signals.h"
 #include "llvm/Support/Threading.h"
 #include "llvm/Support/ThreadLocal.h"
+#endif
+
+#if LLVM_VERSION < 33
 #include "llvm/IntrinsicInst.h"
-#include "llvm/PassRegistry.h"
 #else
-#include "llvm/System/DataTypes.h"
-#include "llvm/System/Host.h"
-#include "llvm/System/Memory.h"
-#include "llvm/System/Mutex.h"
-#include "llvm/System/Signals.h"
-#include "llvm/System/Threading.h"
-#include "llvm/System/ThreadLocal.h"
+#include "llvm/IR/IntrinsicInst.h"
 #endif
 
 #include "llvm/Support/Timer.h"
@@ -88,31 +100,78 @@ extern "C" {
 void LLVMInitializeX86AsmPrinter();
 void LLVMInitializePowerPCAsmPrinter();
 }
-#include "llvm30_compat.h"
-#ifdef LLVM30
-#include "llvm/Support/TargetSelect.h"
-#else
+
+#if LLVM_VERSION < 30
 #include "llvm/Target/TargetSelect.h"
+#else
+#include "llvm/Support/TargetSelect.h"
 #endif
-#include "llvm/Target/TargetData.h"
+
 #include "llvm/Target/TargetOptions.h"
-#include "llvm/Support/TargetFolder.h"
 #include "llvm/Transforms/Scalar.h"
 #include "llvm/Transforms/IPO.h"
 #include "llvm/Transforms/Utils/BasicBlockUtils.h"
-#include <cstdlib>
-#include <csetjmp>
-#include <new>
-#include <cerrno>
-#include <string>
+
+#if LLVM_VERSION < 32
+#include "llvm/Analysis/DebugInfo.h"
+#elif LLVM_VERSION < 35
+#include "llvm/DebugInfo.h"
+#else
+#include "llvm/IR/DebugInfo.h"
+#endif
+
+#if LLVM_VERSION < 32
+#include "llvm/Support/IRBuilder.h"
+#include "llvm/Target/TargetData.h"
+#elif LLVM_VERSION < 33
+#include "llvm/IRBuilder.h"
+#include "llvm/DataLayout.h"
+#else
+#include "llvm/IR/IRBuilder.h"
+#include "llvm/IR/DataLayout.h"
+#endif
+
+#if LLVM_VERSION < 33
+#include "llvm/CallingConv.h"
+#include "llvm/DerivedTypes.h"
+#include "llvm/Function.h"
+#include "llvm/LLVMContext.h"
+#include "llvm/Intrinsics.h"
+#include "llvm/Module.h"
+#else
+#include "llvm/IR/CallingConv.h"
+#include "llvm/IR/DerivedTypes.h"
+#include "llvm/IR/Function.h"
+#include "llvm/IR/LLVMContext.h"
+#include "llvm/IR/Intrinsics.h"
+#include "llvm/IR/Module.h"
+#endif
+
+#if LLVM_VERSION < 34
+#include "llvm/Support/CFG.h"
+#else
+#include "llvm/Analysis/CFG.h"
+#endif
+
+#if LLVM_VERSION >= 35
+#include "llvm/IR/Dominators.h"
+#endif
 
 //#define TIMING
 #undef TIMING
 
 #include "llvm/Config/config.h"
+#ifdef ENABLE_THREADS
 #if !ENABLE_THREADS
 #error "Thread support was explicitly disabled. Cannot continue"
 #endif
+#endif
+
+#ifdef LLVM_ENABLE_THREADS
+#if !LLVM_ENABLE_THREADS
+#error "Thread support was explicitly disabled. Cannot continue"
+#endif
+#endif
 
 #ifdef _GLIBCXX_PARALLEL
 #error "libstdc++ parallel mode is not supported for ClamAV. Please remove -D_GLIBCXX_PARALLEL from CXXFLAGS!"
@@ -131,10 +190,6 @@ void LLVMInitializePowerPCAsmPrinter();
 #include <openssl/ssl.h>
 #include <openssl/err.h>
 
-extern "C" {
-#include "libclamav/crypto.h"
-}
-
 #include "dconf.h"
 #include "clamav.h"
 #include "clambc.h"
@@ -159,18 +214,14 @@ struct cli_bcengine {
 };
 
 extern "C" uint8_t cli_debug_flag;
-#ifdef LLVM29
+#if LLVM_VERSION >= 29
 namespace llvm {
     void initializeRuntimeLimitsPass(PassRegistry&);
 };
 #endif
 namespace {
 
-#ifndef LLVM28
-#define LLVM28
-#endif
-
-#ifdef LLVM28
+#if LLVM_VERSION >= 28
 #define llvm_report_error(x) report_fatal_error(x)
 #define llvm_install_error_handler(x) install_fatal_error_handler(x)
 #define DwarfExceptionHandling JITExceptionHandling
@@ -180,7 +231,7 @@ namespace {
 #define DEFINEPASS(passname) passname() : FunctionPass(&ID)
 #endif
 
-#ifdef LLVM29
+#if LLVM_VERSION >= 29
 #define NORETURN LLVM_ATTRIBUTE_NORETURN
 #endif
 
@@ -274,7 +325,11 @@ static void NORETURN jit_ssp_handler(void)
     jit_exception_handler();
 }
 
+#if LLVM_VERSION < 33
 void llvm_error_handler(void *user_data, const std::string &reason)
+#else
+void llvm_error_handler(void *user_data, const std::string &reason, bool gen_crash_diag = true)
+#endif
 {
     // Output it to stderr, it might exceed the 1k/4k limit of cli_errmsg
     cli_errmsg("[Bytecode JIT]: [LLVM error] %s\n", reason.c_str());
@@ -383,7 +438,11 @@ public:
 	if (!cli_debug_flag)
 	    return;
 	cli_dbgmsg_internal("[Bytecode JIT]: emitted function %s of %ld bytes at %p\n",
+#if LLVM_VERSION < 31
 			    F.getNameStr().c_str(), (long)Size, Code);
+#else
+			    F.getName().str().c_str(), (long)Size, Code);
+#endif
     }
 };
 
@@ -416,10 +475,10 @@ public:
 
 class LLVMTypeMapper {
 private:
-#ifdef LLVM30
-    std::vector<Type*> TypeMap;
-#else
+#if LLVM_VERSION < 30
     std::vector<PATypeHolder> TypeMap;
+#else
+    std::vector<Type*> TypeMap;
 #endif
     LLVMContext &Context;
     unsigned numTypes;
@@ -454,22 +513,22 @@ public:
 	// invalidated, so we must use a TypeHolder to an Opaque type as a
 	// start.
 	for (unsigned i=0;i<count;i++) {
-#ifdef LLVM30
-	    TypeMap.push_back(0);
-#else
+#if LLVM_VERSION < 30
 	    TypeMap.push_back(OpaqueType::get(Context));
+#else
+	    TypeMap.push_back(0);
 #endif
 	}
 	for (unsigned i=0;i<count;i++) {
 	    const struct cli_bc_type *type = &types[i];
 
 	    constType *Ty = buildType(type, types, Hidden, 0);
-#ifdef LLVM30
-	    TypeMap[i] = Ty;
-#else
+#if LLVM_VERSION < 30
 	    // Make the opaque type a concrete type, doing recursive type
 	    // unification if needed.
 	    cast<OpaqueType>(TypeMap[i].get())->refineAbstractTypeTo(Ty);
+#else
+	    TypeMap[i] = Ty;
 #endif
 	}
     }
@@ -521,7 +580,9 @@ public:
 	    return getStatic(ty);
 	ty -= 69;
 	assert(ty < numTypes && "TypeID out of range");
-#ifdef LLVM30
+#if LLVM_VERSION < 30
+	return TypeMap[ty].get();
+#else
 	Type *Ty = TypeMap[ty];
 	if (Ty)
 	    return Ty;
@@ -529,8 +590,6 @@ public:
 	Ty = buildType(&types[ty], types, Hidden, 1);
 	TypeMap[ty] = Ty;
 	return Ty;
-#else
-	return TypeMap[ty].get();
 #endif
     }
 };
@@ -589,7 +648,7 @@ class RuntimeLimits : public FunctionPass {
 public:
     static char ID;
     DEFINEPASS(RuntimeLimits) {
-#ifdef LLVM29
+#if LLVM_VERSION >= 29
 	PassRegistry &Registry = *PassRegistry::getPassRegistry();
 	initializeRuntimeLimitsPass(Registry);
 #endif
@@ -611,7 +670,11 @@ public:
 	}
 	BBSetTy  needsTimeoutCheck;
 	BBMapTy BBMap;
+#if LLVM_VERSION < 35
 	DominatorTree &DT = getAnalysis<DominatorTree>();
+#else
+	DominatorTree &DT = getAnalysis<DominatorTreeWrapperPass>().getDomTree();
+#endif
 	for (Function::iterator I=F.begin(),E=F.end(); I != E; ++I) {
 	    BasicBlock *BB = &*I;
 	    unsigned apicalls = 0;
@@ -676,13 +739,18 @@ public:
         CallInst* AbrtC = CallInst::Create(func_abort, "", AbrtBB);
         AbrtC->setCallingConv(CallingConv::C);
         AbrtC->setTailCall(true);
+#if LLVM_VERSION < 32
         AbrtC->setDoesNotReturn(true);
         AbrtC->setDoesNotThrow(true);
+#else
+        AbrtC->setDoesNotReturn();
+        AbrtC->setDoesNotThrow();
+#endif
         new UnreachableInst(F.getContext(), AbrtBB);
 	IRBuilder<false> Builder(F.getContext());
 
 	Value *Flag = F.arg_begin();
-#ifndef LLVM30
+#if LLVM_VERSION < 30
 	Function *LSBarrier = Intrinsic::getDeclaration(F.getParent(),
 							Intrinsic::memory_barrier);
 	Value *MBArgs[] = {
@@ -702,11 +770,11 @@ public:
 	     E=needsTimeoutCheck.end(); I != E; ++I) {
 	    BasicBlock *BB = *I;
 	    Builder.SetInsertPoint(BB, BB->getTerminator());
-#ifdef LLVM30
-	    Builder.CreateFence(Release);
-#else
+#if LLVM_VERSION < 30
 	    // store-load barrier: will be a no-op on x86 but not other arches
 	    Builder.CreateCall(LSBarrier, ARRAYREF(Value*, MBArgs, MBArgs+5));
+#else
+	    Builder.CreateFence(Release);
 #endif
 	    // Load Flag that tells us we timed out (first byte in bc_ctx)
 	    Value *Cond = Builder.CreateLoad(Flag, true);
@@ -733,7 +801,11 @@ public:
       AU.setPreservesAll();
       AU.addRequired<LoopInfo>();
       AU.addRequired<ScalarEvolution>();
+#if LLVM_VERSION < 35
       AU.addRequired<DominatorTree>();
+#else
+      AU.addRequired<DominatorTreeWrapperPass>();
+#endif
     }
 };
 char RuntimeLimits::ID;
@@ -1027,7 +1099,11 @@ public:
 	: bc(bc), M(M), Context(M->getContext()), EE(EE),
 	PM(PM),PMUnsigned(PMUnsigned), TypeMap(), apiFuncs(apiFuncs),apiMap(apiMap),
 	compiledFunctions(cFuncs), BytecodeID("bc"+Twine(bc->id)),
+#if LLVM_VERSION < 32
 	Folder(EE->getTargetData()), Builder(Context, Folder), Values(), CF(CF) {
+#else
+	Folder(EE->getDataLayout()), Builder(Context, Folder), Values(), CF(CF) {
+#endif
 
 	for (unsigned i=0;i<cli_apicall_maxglobal - _FIRST_GLOBAL;i++) {
 	    unsigned id = cli_globals[i].globalid;
@@ -1037,7 +1113,7 @@ public:
 	numArgs = 0;
     }
 
-#ifndef LLVM30
+#if LLVM_VERSION < 30
     template <typename InputIterator>
 #endif
     Value* createGEP(Value *Base, constType *ETy, ARRAYREFPARAM(Value*,InputIterator Start,InputIterator End,ARef)) {
@@ -1054,10 +1130,10 @@ public:
 		ostr << " base: " << *Base << ";";
 		Base->getType()->print(ostr);
 		ostr << "\n indices: ";
-#ifdef LLVM30
-		for (ArrayRef<Value*>::iterator I=ARef.begin(); I != ARef.end(); I++) {
-#else
+#if LLVM_VERSION < 30
 		for (InputIterator I=Start; I != End; I++) {
+#else
+		for (ArrayRef<Value*>::iterator I=ARef.begin(); I != ARef.end(); I++) {
 #endif
 		    ostr << **I << ", ";
 		}
@@ -1071,7 +1147,7 @@ public:
 	return Builder.CreateGEP(Base,ARRAYREFP(Start,End,ARef));
     }
 
-#ifndef LLVM30
+#if LLVM_VERSION < 30
     template <typename InputIterator>
 #endif
     bool createGEP(unsigned dest, Value *Base, ARRAYREFPARAM(Value*,InputIterator Start,InputIterator End,ARef)) {
@@ -1128,7 +1204,7 @@ public:
 	    // Have an alloca -> some instruction uses its address otherwise
 	    // mem2reg would have converted it to an SSA register.
 	    // Enable stack protector for this function.
-#ifndef LLVM29
+#if LLVM_VERSION < 29
 	    // LLVM 2.9 has broken SSP, it does a 'mov 0x28, $rax', which tries
 	    // to read from the address 0x28 and crashes
 	    F->addFnAttr(Attribute::StackProtectReq);
@@ -1137,7 +1213,7 @@ public:
 	// always add stackprotect attribute (bb #2239), so we know this
 	// function was verified. If there is no alloca it won't actually add
 	// stack protector in emitted code so this won't slow down the app.
-#ifndef LLVM29
+#if LLVM_VERSION < 29
 	F->addFnAttr(Attribute::StackProtect);
 #endif
     }
@@ -1158,7 +1234,11 @@ public:
 	    }
 	    V = SI->getOperand(0);
 	}
+#if LLVM_VERSION < 32
 	if (EE->getTargetData()->getPointerSize() == 8) {
+#else
+	if (EE->getDataLayout()->getPointerSize() == 8) {
+#endif
 	    // eliminate useless trunc, GEP can take i64 too
 	    if (TruncInst *I = dyn_cast<TruncInst>(V)) {
 		Value *Src = I->getOperand(0);
@@ -1233,7 +1313,12 @@ public:
 	    Functions[j]->setLinkage(GlobalValue::InternalLinkage);
 #ifdef C_LINUX
 	    /* bb #2270, this should really be fixed either by LLVM or GCC.*/
+#if LLVM_VERSION < 32
 	    Functions[j]->addFnAttr(Attribute::constructStackAlignmentFromInt(16));
+#else
+	// TODO: How does this translate?
+//	    Functions[j]->addFnAttr(Attribute::StackAlignment);
+#endif
 #endif
 	}
 	constType *I32Ty = Type::getInt32Ty(Context);
@@ -1510,6 +1595,9 @@ public:
 			case OP_BC_ICMP_SLT:
 			    Store(inst->dest, Builder.CreateICmpSLT(Op0, Op1));
 			    break;
+			case OP_BC_ICMP_SLE:
+			    Store(inst->dest, Builder.CreateICmpSLE(Op0, Op1));
+			    break;
 			case OP_BC_SELECT:
 			    Store(inst->dest, Builder.CreateSelect(Op0, Op1, Op2));
 			    break;
@@ -1534,7 +1622,11 @@ public:
 			    }
 			    CallInst *CI = Builder.CreateCall(DestF, ARRAYREF(Value*, args.begin(), args.end()));
 			    CI->setCallingConv(CallingConv::Fast);
+#if LLVM_VERSION < 32
 			    CI->setDoesNotThrow(true);
+#else
+			    CI->setDoesNotThrow();
+#endif
 			    if (CI->getType()->getTypeID() != Type::VoidTyID)
 				Store(inst->dest, CI);
 			    break;
@@ -1555,7 +1647,11 @@ public:
 				args.push_back(convertOperand(func, DestF->getFunctionType()->getParamType(a+1), op));
 			    }
 			    CallInst *CI = Builder.CreateCall(DestF, ARRAYREFVECTOR(Value*, args));
+#if LLVM_VERSION < 32
 			    CI->setDoesNotThrow(true);
+#else
+			    CI->setDoesNotThrow();
+#endif
 			    Store(inst->dest, CI);
 			    }
 			    break;
@@ -1627,14 +1723,14 @@ public:
 			    Dst = Builder.CreatePointerCast(Dst, PointerType::getUnqual(Type::getInt8Ty(Context)));
 			    Value *Val = convertOperand(func, Type::getInt8Ty(Context), inst->u.three[1]);
 			    Value *Len = convertOperand(func, Type::getInt32Ty(Context), inst->u.three[2]);
-#ifdef LLVM30
+#if LLVM_VERSION < 29
+			    CallInst *c = Builder.CreateCall4(CF->FMemset, Dst, Val, Len,
+								ConstantInt::get(Type::getInt32Ty(Context), 1));
+#else
 			    CallInst *c = Builder.CreateCall5(CF->FMemset, Dst, Val, Len,
 								ConstantInt::get(Type::getInt32Ty(Context), 1),
 								ConstantInt::get(Type::getInt1Ty(Context), 0)
 								);
-#else
-			    CallInst *c = Builder.CreateCall4(CF->FMemset, Dst, Val, Len,
-								ConstantInt::get(Type::getInt32Ty(Context), 1));
 #endif
 			    c->setTailCall(true);
 			    c->setDoesNotThrow();
@@ -1648,14 +1744,14 @@ public:
 			    Value *Src = convertOperand(func, inst, inst->u.three[1]);
 			    Src = Builder.CreatePointerCast(Src, PointerType::getUnqual(Type::getInt8Ty(Context)));
 			    Value *Len = convertOperand(func, Type::getInt32Ty(Context), inst->u.three[2]);
-#ifdef LLVM30
+#if LLVM_VERSION < 29
+			    CallInst *c = Builder.CreateCall4(CF->FMemcpy, Dst, Src, Len,
+								ConstantInt::get(Type::getInt32Ty(Context), 1));
+#else
 			    CallInst *c = Builder.CreateCall5(CF->FMemcpy, Dst, Src, Len,
 								ConstantInt::get(Type::getInt32Ty(Context), 1),
 								ConstantInt::get(Type::getInt1Ty(Context), 0)
 								);
-#else
-			    CallInst *c = Builder.CreateCall4(CF->FMemcpy, Dst, Src, Len,
-								ConstantInt::get(Type::getInt32Ty(Context), 1));
 #endif
 			    c->setTailCall(true);
 			    c->setDoesNotThrow();
@@ -1669,13 +1765,13 @@ public:
 			    Value *Src = convertOperand(func, inst, inst->u.three[1]);
 			    Src = Builder.CreatePointerCast(Src, PointerType::getUnqual(Type::getInt8Ty(Context)));
 			    Value *Len = convertOperand(func, Type::getInt32Ty(Context), inst->u.three[2]);
-#ifdef LLVM30
+#if LLVM_VERSION < 29
+			    CallInst *c = Builder.CreateCall4(CF->FMemmove, Dst, Src, Len,
+								ConstantInt::get(Type::getInt32Ty(Context), 1));
+#else
 			    CallInst *c = Builder.CreateCall5(CF->FMemmove, Dst, Src, Len,
 								ConstantInt::get(Type::getInt32Ty(Context), 1),
 								ConstantInt::get(Type::getInt1Ty(Context), 0));
-#else
-			    CallInst *c = Builder.CreateCall4(CF->FMemmove, Dst, Src, Len,
-								ConstantInt::get(Type::getInt32Ty(Context), 1));
 #endif
 			    c->setTailCall(true);
 			    c->setDoesNotThrow();
@@ -1688,7 +1784,11 @@ public:
 			    Dst = Builder.CreatePointerCast(Dst, PointerType::getUnqual(Type::getInt8Ty(Context)));
 			    Value *Src = convertOperand(func, inst, inst->u.three[1]);
 			    Src = Builder.CreatePointerCast(Src, PointerType::getUnqual(Type::getInt8Ty(Context)));
+#if LLVM_VERSION < 32
 			    Value *Len = convertOperand(func, EE->getTargetData()->getIntPtrType(Context), inst->u.three[2]);
+#else
+			    Value *Len = convertOperand(func, EE->getDataLayout()->getIntPtrType(Context), inst->u.three[2]);
+#endif
 			    CallInst *c = Builder.CreateCall3(CF->FRealmemcmp, Dst, Src, Len);
 			    c->setTailCall(true);
 			    c->setDoesNotThrow();
@@ -1713,7 +1813,11 @@ public:
 			    {
 				CallInst *C = Builder.CreateCall(CF->FBSwap16, convertOperand(func, inst, inst->u.unaryop));
 				C->setTailCall(true);
+#if LLVM_VERSION < 32
 				C->setDoesNotThrow(true);
+#else
+				C->setDoesNotThrow();
+#endif
 				Store(inst->dest, C);
 				break;
 			    }
@@ -1721,7 +1825,11 @@ public:
 			    {
 				CallInst *C = Builder.CreateCall(CF->FBSwap32, convertOperand(func, inst, inst->u.unaryop));
 				C->setTailCall(true);
+#if LLVM_VERSION < 32
 				C->setDoesNotThrow(true);
+#else
+				C->setDoesNotThrow();
+#endif
 				Store(inst->dest, C);
 				break;
 			    }
@@ -1729,7 +1837,11 @@ public:
 			    {
 				CallInst *C = Builder.CreateCall(CF->FBSwap64, convertOperand(func, inst, inst->u.unaryop));
 				C->setTailCall(true);
+#if LLVM_VERSION < 32
 				C->setDoesNotThrow(true);
+#else
+				C->setDoesNotThrow();
+#endif
 				Store(inst->dest, C);
 				break;
 			    }
@@ -1762,7 +1874,11 @@ public:
 
 	    // If successful so far, run verifyFunction
 	    if (!broken) {
+#if LLVM_VERSION < 35
 		if (verifyFunction(*F, PrintMessageAction)) {
+#else
+		if (verifyFunction(*F, &errs())) {
+#endif
 		    // verification failed
 		    broken = true;
 		    cli_warnmsg("[Bytecode JIT]: Verification failed\n");
@@ -1834,7 +1950,11 @@ public:
 	// entrypoint can only be C, emit wrapper
 	Function *F = Function::Create(Functions[0]->getFunctionType(),
 				       Function::ExternalLinkage,
+#if LLVM_VERSION < 33
 				       Functions[0]->getName()+"_wrap", M);
+#else
+				       Functions[0]->getName().str()+"_wrap", M);
+#endif
 	F->setDoesNotThrow();
 	BasicBlock *BB = BasicBlock::Create(Context, "", F);
 	std::vector<Value*> Args;
@@ -1847,7 +1967,11 @@ public:
 	ReturnInst::Create(Context, CI, BB);
 
 	delete [] Functions;
+#if LLVM_VERSION < 35
 	if (verifyFunction(*F, PrintMessageAction))
+#else
+	if (verifyFunction(*F, &errs()))
+#endif
 	    return 0;
 
 /*			DEBUG(errs() << "Generating code\n");
@@ -1893,8 +2017,11 @@ static void addFunctionProtos(struct CommonFunctions *CF, ExecutionEngine *EE, M
 					  "clamjit.fail", M);
     CF->FHandler->setDoesNotReturn();
     CF->FHandler->setDoesNotThrow();
+#if LLVM_VERSION == 32
+    CF->FHandler->addFnAttr(Attributes::NoInline);
+#else
     CF->FHandler->addFnAttr(Attribute::NoInline);
-
+#endif
     EE->addGlobalMapping(CF->FHandler, (void*)(intptr_t)jit_exception_handler);
     EE->InstallLazyFunctionCreator(noUnknownFunctions);
     EE->getPointerToFunction(CF->FHandler);
@@ -1904,50 +2031,62 @@ static void addFunctionProtos(struct CommonFunctions *CF, ExecutionEngine *EE, M
     args.push_back(Type::getInt8Ty(Context));
     args.push_back(Type::getInt32Ty(Context));
     args.push_back(Type::getInt32Ty(Context));
-#ifdef LLVM30
+#if LLVM_VERSION >= 29
     args.push_back(Type::getInt1Ty(Context));
 #endif
     FunctionType* FuncTy_3 = FunctionType::get(Type::getVoidTy(Context),
 					       args, false);
     CF->FMemset = Function::Create(FuncTy_3, GlobalValue::ExternalLinkage,
-#ifdef LLVM30
-					 "llvm.memset.p0i8.i32",
+#if LLVM_VERSION < 29
+                                   "llvm.memset.i32",
 #else
-					 "llvm.memset.i32",
+                                   "llvm.memset.p0i8.i32",
 #endif
-					 M);
+                                   M);
     CF->FMemset->setDoesNotThrow();
+#if LLVM_VERSION < 32
     CF->FMemset->setDoesNotCapture(1, true);
+#else
+    CF->FMemset->setDoesNotCapture(1);
+#endif
 
     args.clear();
     args.push_back(PointerType::getUnqual(Type::getInt8Ty(Context)));
     args.push_back(PointerType::getUnqual(Type::getInt8Ty(Context)));
     args.push_back(Type::getInt32Ty(Context));
     args.push_back(Type::getInt32Ty(Context));
-#ifdef LLVM30
+#if LLVM_VERSION >= 29
     args.push_back(Type::getInt1Ty(Context));
 #endif
     FunctionType* FuncTy_4 = FunctionType::get(Type::getVoidTy(Context),
 					       args, false);
     CF->FMemmove = Function::Create(FuncTy_4, GlobalValue::ExternalLinkage,
-#ifdef LLVM30
-					  "llvm.memmove.p0i8.i32",
+#if LLVM_VERSION < 29
+                                    "llvm.memmove.i32",
 #else
-					  "llvm.memmove.i32",
+                                    "llvm.memmove.p0i8.i32",
 #endif
-					  M);
+                                    M);
     CF->FMemmove->setDoesNotThrow();
+#if LLVM_VERSION < 32
     CF->FMemmove->setDoesNotCapture(1, true);
+#else
+    CF->FMemmove->setDoesNotCapture(1);
+#endif
 
     CF->FMemcpy = Function::Create(FuncTy_4, GlobalValue::ExternalLinkage,
-#ifdef LLVM30
-					 "llvm.memcpy.p0i8.p0i8.i32",
+#if LLVM_VERSION < 29
+                                   "llvm.memcpy.i32",
 #else
-					 "llvm.memcpy.i32",
+                                   "llvm.memcpy.p0i8.p0i8.i32",
 #endif
-					 M);
+                                   M);
     CF->FMemcpy->setDoesNotThrow();
+#if LLVM_VERSION < 32
     CF->FMemcpy->setDoesNotCapture(1, true);
+#else
+    CF->FMemcpy->setDoesNotCapture(1);
+#endif
 
     args.clear();
     args.push_back(Type::getInt16Ty(Context));
@@ -1987,7 +2126,11 @@ static void addFunctionProtos(struct CommonFunctions *CF, ExecutionEngine *EE, M
     args.clear();
     args.push_back(PointerType::getUnqual(Type::getInt8Ty(Context)));
     args.push_back(PointerType::getUnqual(Type::getInt8Ty(Context)));
+#if LLVM_VERSION < 32
     args.push_back(EE->getTargetData()->getIntPtrType(Context));
+#else
+    args.push_back(EE->getDataLayout()->getIntPtrType(Context));
+#endif
     FuncTy_5 = FunctionType::get(Type::getInt32Ty(Context),
 				 args, false);
     CF->FRealmemcmp = Function::Create(FuncTy_5, GlobalValue::ExternalLinkage, "memcmp", M);
@@ -1996,11 +2139,15 @@ static void addFunctionProtos(struct CommonFunctions *CF, ExecutionEngine *EE, M
 }
 
 }
-#ifdef LLVM29
+#if LLVM_VERSION >= 29
 INITIALIZE_PASS_BEGIN(RuntimeLimits, "rl", "Runtime Limits", false, false)
 INITIALIZE_PASS_DEPENDENCY(LoopInfo)
 INITIALIZE_PASS_DEPENDENCY(ScalarEvolution)
+#if LLVM_VERSION < 35
 INITIALIZE_PASS_DEPENDENCY(DominatorTree)
+#else
+INITIALIZE_PASS_DEPENDENCY(DominatorTreeWrapperPass)
+#endif
 INITIALIZE_PASS_END(RuntimeLimits, "rl" ,"Runtime Limits", false, false)
 #endif
 
@@ -2210,16 +2357,27 @@ static void setGuard(unsigned char* guardbuf)
     char salt[48];
     memcpy(salt, name_salt, 16);
     for(unsigned i = 16; i < 48; i++)
-        salt[i] = cli_rndnum(255);
+	salt[i] = cli_rndnum(255);
 
     cl_hash_data("md5", salt, 48, guardbuf, NULL);
 }
-
+#if LLVM_VERSION < 32
 static void addFPasses(FunctionPassManager &FPM, bool trusted, const TargetData *TD)
+#elif LLVM_VERSION < 35
+static void addFPasses(FunctionPassManager &FPM, bool trusted, const DataLayout *TD)
+#else
+static void addFPasses(FunctionPassManager &FPM, bool trusted, const Module *M)
+#endif
 {
     // Set up the optimizer pipeline.  Start with registering info about how
     // the target lays out data structures.
+#if LLVM_VERSION < 32
     FPM.add(new TargetData(*TD));
+#elif LLVM_VERSION < 35
+    FPM.add(new DataLayout(*TD));
+#else
+    FPM.add(new DataLayoutPass(M));
+#endif
     // Promote allocas to registers.
     FPM.add(createPromoteMemoryToRegisterPass());
     FPM.add(new BrSimplifier());
@@ -2241,6 +2399,24 @@ int cli_bytecode_prepare_jit(struct cli_all_bc *bcs)
 	// Create the JIT.
 	std::string ErrorMsg;
 	EngineBuilder builder(M);
+
+#if LLVM_VERSION >= 31
+	TargetOptions Options;
+#ifdef CL_DEBUG
+	//disable this for now, it leaks
+	Options.JITEmitDebugInfo = false;
+//	Options.JITEmitDebugInfo = true;
+#else
+	Options.JITEmitDebugInfo = false;
+#endif
+#if LLVM_VERSION < 34
+	Options.DwarfExceptionHandling = false;
+#else
+	// TODO: How to do this now?
+#endif
+	builder.setTargetOptions(Options);
+#endif
+
 	builder.setErrorStr(&ErrorMsg);
 	builder.setEngineKind(EngineKind::JIT);
 	builder.setOptLevel(CodeGenOpt::Default);
@@ -2265,11 +2441,27 @@ int cli_bytecode_prepare_jit(struct cli_all_bc *bcs)
 	addFunctionProtos(&CF, EE, M);
 
 	FunctionPassManager OurFPM(M), OurFPMUnsigned(M);
+#if LLVM_VERSION < 32
 	M->setDataLayout(EE->getTargetData()->getStringRepresentation());
+#else
+	M->setDataLayout(EE->getDataLayout()->getStringRepresentation());
+#endif
+#if LLVM_VERSION < 31
 	M->setTargetTriple(sys::getHostTriple());
-
+#else
+	M->setTargetTriple(sys::getDefaultTargetTriple());
+#endif
+#if LLVM_VERSION < 32
 	addFPasses(OurFPM, true, EE->getTargetData());
 	addFPasses(OurFPMUnsigned, false, EE->getTargetData());
+#elif LLVM_VERSION < 35
+	addFPasses(OurFPM, true, EE->getDataLayout());
+	addFPasses(OurFPMUnsigned, false, EE->getDataLayout());
+#else
+	addFPasses(OurFPM, true, M);
+	addFPasses(OurFPMUnsigned, false, M);
+#endif
+
 
 	//TODO: create a wrapper that calls pthread_getspecific
 	unsigned maxh = cli_globals[0].offset + sizeof(struct cli_bc_hooks);
@@ -2376,7 +2568,13 @@ int cli_bytecode_prepare_jit(struct cli_all_bc *bcs)
 	    }
 	}
 	PassManager PM;
+#if LLVM_VERSION < 32
 	PM.add(new TargetData(*EE->getTargetData()));
+#elif LLVM_VERSION < 35
+	PM.add(new DataLayout(*EE->getDataLayout()));
+#else
+	PM.add(new DataLayoutPass(M));
+#endif
 	// TODO: only run this on the untrusted bytecodes, not all of them...
 	if (has_untrusted)
 	    PM.add(createClamBCRTChecks());
@@ -2434,18 +2632,31 @@ int cli_bytecode_prepare_jit(struct cli_all_bc *bcs)
 int bytecode_init(void)
 {
     // If already initialized return
+#if LLVM_VERSION < 35
     if (llvm_is_multithreaded()) {
 	cli_warnmsg("bytecode_init: already initialized\n");
 	return CL_EARG;
     }
+#else
+    if (!LLVMIsMultithreaded()) {
+        cli_warnmsg("bytecode_init: LLVM is compiled without multithreading support\n");
+    }
+#endif
+
     llvm_install_error_handler(llvm_error_handler);
 #ifdef CL_DEBUG
     sys::PrintStackTraceOnErrorSignal();
+#if LLVM_VERSION >= 34
+    llvm::EnablePrettyStackTrace();
+#endif
 #else
+#if LLVM_VERSION < 34
     llvm::DisablePrettyStackTrace = true;
+#endif
 #endif
     atexit(do_shutdown);
 
+#if LLVM_VERSION < 31
 #ifdef CL_DEBUG
     //disable this for now, it leaks
     llvm::JITEmitDebugInfo = false;
@@ -2454,7 +2665,12 @@ int bytecode_init(void)
     llvm::JITEmitDebugInfo = false;
 #endif
     llvm::DwarfExceptionHandling = false;
+#endif
+#if LLVM_VERSION < 33
     llvm_start_multithreaded();
+#else
+    // This is now deprecated/useless: Multi-threading can only be enabled/disabled with the compile time define LLVM_ENABLE_THREADS in LLVM.
+#endif
 
     // If we have a native target, initialize it to ensure it is linked in and
     // usable by the JIT.
@@ -2464,7 +2680,11 @@ int bytecode_init(void)
     InitializeAllTargets();
 #endif
 
+#if LLVM_VERSION < 35
     if (!llvm_is_multithreaded()) {
+#else
+    if (!LLVMIsMultithreaded()) {
+#endif
 	//TODO:cli_dbgmsg
 	DEBUG(errs() << "WARNING: ClamAV JIT built w/o atomic builtins\n"
 	      << "On x86 for best performance ClamAV should be built for i686, not i386!\n");
@@ -2534,7 +2754,9 @@ void cli_bytecode_debug_printsrc(const struct cli_bc_ctx *ctx)
     if (I == LinePrinter.files.end()) {
 	lines = new linesTy;
 	std::string ErrorMessage;
-#ifdef LLVM29
+#if LLVM_VERSION < 29
+	lines->buffer = MemoryBuffer::getFile(path, &ErrorMessage);
+#elif LLVM_VERSION < 35
 	OwningPtr<MemoryBuffer> File;
 	error_code ec = MemoryBuffer::getFile(path, File);
 	if (ec) {
@@ -2543,7 +2765,14 @@ void cli_bytecode_debug_printsrc(const struct cli_bc_ctx *ctx)
 	} else
 	    lines->buffer = File.take();
 #else
-	lines->buffer = MemoryBuffer::getFile(path, &ErrorMessage);
+	ErrorOr<std::unique_ptr<MemoryBuffer>> FileOrErr = MemoryBuffer::getFile(path);
+	if (!FileOrErr) {
+		// TODO: How to handle ErrorMessage?
+		lines->buffer = 0;
+	}
+	else {
+		lines->buffer = FileOrErr.get().release();
+	}
 #endif
 	if (!lines->buffer) {
 	    errs() << "Unable to open file '" << path << "'\n";
@@ -2577,7 +2806,7 @@ void cli_bytecode_debug_printsrc(const struct cli_bc_ctx *ctx)
     }
     assert(ctx->line < lines->linev.size());
 
-#ifndef LLVM28
+#if LLVM_VERSION < 28
     int line = (int)ctx->line ? (int)ctx->line : -1;
     int col = (int)ctx->col ? (int)ctx->col : -1;
     //TODO: print this ourselves, instead of using SMDiagnostic
@@ -2612,14 +2841,18 @@ namespace ClamBCModule {
 void stop(const char *msg, llvm::Function* F, llvm::Instruction* I)
 {
     if (F && F->hasName()) {
+#if LLVM_VERSION < 31
 	cli_warnmsg("[Bytecode JIT] in function %s: %s", F->getNameStr().c_str(), msg);
+#else
+	cli_warnmsg("[Bytecode JIT] in function %s: %s", F->getName().str().c_str(), msg);
+#endif
     } else {
 	cli_warnmsg("[Bytecode JIT] %s", msg);
     }
 }
 }
 
-#ifdef LLVM29
+#if LLVM_VERSION >= 29
 static Value *findDbgGlobalDeclare(GlobalVariable *V) {
   const Module *M = V->getParent();
   NamedMDNode *NMD = M->getNamedMetadata("llvm.dbg.gv");
@@ -2679,8 +2912,16 @@ static const DbgDeclareInst *findDbgDeclare(const Value *V) {
 static bool getLocationInfo(const Value *V, std::string &DisplayName,
                             std::string &Type, unsigned &LineNo,
                             std::string &File, std::string &Dir) {
+#if LLVM_VERSION < 33
   DICompileUnit Unit;
+#else
+  StringRef G;
+  StringRef H;
+#endif
+#if LLVM_VERSION < 35
   DIType TypeD;
+#endif
+  StringRef T;
 
   if (GlobalVariable *GV = dyn_cast<GlobalVariable>(const_cast<Value*>(V))) {
     Value *DIGV = findDbgGlobalDeclare(GV);
@@ -2691,8 +2932,17 @@ static bool getLocationInfo(const Value *V, std::string &DisplayName,
     if (!D.empty())
       DisplayName = D;
     LineNo = Var.getLineNumber();
+#if LLVM_VERSION < 33
     Unit = Var.getCompileUnit();
+#else
+    G = Var.getFilename();
+    H = Var.getDirectory();
+#endif
+#if LLVM_VERSION < 35
     TypeD = Var.getType();
+#else
+    T = Var.getType().getName();
+#endif
   } else if (Function *F = dyn_cast<Function>(const_cast<Value*>(V))){
     Value *DIF = findDbgSubprogramDeclare(F);
     if (!DIF) return false;
@@ -2702,8 +2952,17 @@ static bool getLocationInfo(const Value *V, std::string &DisplayName,
     if (!D.empty())
       DisplayName = D;
     LineNo = Var.getLineNumber();
+#if LLVM_VERSION < 33
     Unit = Var.getCompileUnit();
+#else
+    G = Var.getFilename();
+    H = Var.getDirectory();
+#endif
+#if LLVM_VERSION < 35
     TypeD = Var.getType();
+#else
+    T = Var.getType().getName();
+#endif
   } else {
     const DbgDeclareInst *DDI = findDbgDeclare(V);
     if (!DDI) return false;
@@ -2713,19 +2972,33 @@ static bool getLocationInfo(const Value *V, std::string &DisplayName,
     if (!D.empty())
       DisplayName = D;
     LineNo = Var.getLineNumber();
+#if LLVM_VERSION < 33
     Unit = Var.getCompileUnit();
+#else
+    // getFilename and getDirectory are not defined
+    G = StringRef();
+    H = StringRef();
+#endif
+#if LLVM_VERSION < 35
     TypeD = Var.getType();
+#else
+    T = Var.getType().getName();
+#endif
   }
 
-  StringRef T = TypeD.getName();
+#if LLVM_VERSION < 35
+  T = TypeD.getName();
+#endif
   if (!T.empty())
     Type = T;
-  StringRef F = Unit.getFilename();
-  if (!F.empty())
-    File = F;
-  StringRef D = Unit.getDirectory();
-  if (!D.empty())
-    Dir = D;
+#if LLVM_VERSION < 33
+  StringRef G = Unit.getFilename();
+  StringRef H = Unit.getDirectory();
+#endif
+  if (!G.empty())
+    File = G;
+  if (!H.empty())
+    Dir = H;
   return true;
 }
 #endif
