@@ -1,6 +1,7 @@
 /*
  *  A fast filter for static patterns.
  *
+ *  Copyright (C) 2015 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2008 Sourcefire, Inc.
  *
  *  Authors: Török Edvin
@@ -279,7 +280,7 @@ static inline unsigned char spec_ith_char(const struct char_spec *spec, unsigned
 	if (alt) {
 		assert (alt->type == 1);
 		assert (i < alt->num);
-		return alt->str[i];
+		return (alt->alt).byte[i];
 	}
 	return i;
 }
@@ -433,10 +434,10 @@ int  filter_add_acpatt(struct filter *m, const struct cli_ac_patt *pat)
 
 	struct choice choices[MAX_CHOICES];
 	unsigned choices_cnt = 0;
-	unsigned prefix_len = pat->prefix_length;
+	unsigned prefix_len = pat->prefix_length[0];
 	unsigned speci;
 
-	j = MIN(prefix_len + pat->length, MAXPATLEN);
+	j = MIN(prefix_len + pat->length[0], MAXPATLEN);
 	for(i=0;i<j;i++) {
 		const uint16_t p = i < prefix_len ? pat->prefix[i] : pat->pattern[i - prefix_len];
 		if ((p&CLI_MATCH_WILDCARD) != CLI_MATCH_CHAR)
