@@ -1,4 +1,5 @@
 /*
+ *  Copyright (C) 2015 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2011 Sourcefire, Inc.
  *
  *  Authors: aCaB <acab@clamav.net>
@@ -118,6 +119,7 @@ static char *iso_string(iso9660_t *iso, const void *src, unsigned int len) {
 static int iso_parse_dir(iso9660_t *iso, unsigned int block, unsigned int len) {
     cli_ctx *ctx = iso->ctx;
     int ret = CL_CLEAN;
+    int viruses_found = 0;
 
     if(len < 34) {
 	cli_dbgmsg("iso_parse_dir: Directory too small, skipping\n");
@@ -182,10 +184,13 @@ static int iso_parse_dir(iso9660_t *iso, unsigned int block, unsigned int len) {
 	    filesz = cli_readint32(dir+10);
 
 	    cli_dbgmsg("iso_parse_dir: %s '%s': off %x - size %x - flags %x - unit size %x - gap size %x - volume %u\n", (dir[25] & 2) ? "Directory" : "File", iso->buf, fileoff, filesz, dir[25], dir[26], dir[27], cli_readint32(&dir[28]) & 0xffff);
-	    if(cli_matchmeta(ctx, iso->buf, filesz, filesz, 0, 0, 0, NULL) == CL_VIRUS) {
-		ret = CL_VIRUS;
-		break;
-	    }
+            ret = cli_matchmeta(ctx, iso->buf, filesz, filesz, 0, 0, 0, NULL);
+            if (ret == CL_VIRUS) {
+                viruses_found = 1;
+                if (!SCAN_ALL)
+                    break;
+                ret = CL_CLEAN;
+            }
 
 	    if(dir[26] || dir[27])
 		cli_dbgmsg("iso_parse_dir: Skipping interleaved file\n");
@@ -199,6 +204,12 @@ static int iso_parse_dir(iso9660_t *iso, unsigned int block, unsigned int len) {
 		    else
 			ret = iso_scan_file(iso, fileoff, filesz);
 		}
+                if (ret == CL_VIRUS) {
+                    viruses_found = 1;
+                    if (!SCAN_ALL)
+                        break;
+                    ret = CL_CLEAN;
+                }
 	    }
 	    dirsz -= entrysz;
 	    dir += entrysz;
@@ -206,6 +217,8 @@ static int iso_parse_dir(iso9660_t *iso, unsigned int block, unsigned int len) {
 
 	fmap_unneed_ptr(*ctx->fmap, dir_orig, iso->blocksz);
     }
+    if (viruses_found == 1)
+        return CL_VIRUS;
     return ret;
 }
 
