@@ -1,6 +1,7 @@
 /*
  *  Javascript normalizer.
  *
+ *  Copyright (C) 2015 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2008 Sourcefire, Inc.
  *
  *  Authors: Török Edvin
@@ -525,7 +526,8 @@ static int replace_token_range(struct tokens *dst, size_t start, size_t end, con
 {
 	const size_t len = with ? with->cnt : 0;
 	size_t i;
-	cli_dbgmsg(MODULE "Replacing tokens %lu - %lu with %lu tokens\n",start, end, len);
+	cli_dbgmsg(MODULE "Replacing tokens %lu - %lu with %lu tokens\n", (unsigned long)start,
+                   (unsigned long)end, (unsigned long)len);
 	if(start >= dst->cnt || end > dst->cnt)
 		return -1;
 	for(i=start;i<end;i++) {
@@ -547,7 +549,7 @@ static int append_tokens(struct tokens *dst, const struct tokens *src)
 		return CL_ENULLARG;
 	if(tokens_ensure_capacity(dst, dst->cnt + src->cnt))
 		return CL_EMEM;
-	cli_dbgmsg(MODULE "Appending %lu tokens\n", src->cnt);
+	cli_dbgmsg(MODULE "Appending %lu tokens\n", (unsigned long)(src->cnt));
 	memcpy(&dst->data[dst->cnt], src->data, src->cnt * sizeof(dst->data[0]));
 	dst->cnt += src->cnt;
 	return CL_SUCCESS;
@@ -1110,7 +1112,7 @@ void cli_js_process_buffer(struct parser_state *state, const char *buf, size_t n
 				TOKEN_SET(&val, scope, state->current);
 				break;
 			case TOK_StringLiteral:
-				if(state->tokens.cnt > 0 && state->tokens.data[state->tokens.cnt-1].type == TOK_PLUS) {
+				if(state->tokens.cnt > 1 && state->tokens.data[state->tokens.cnt-1].type == TOK_PLUS) {
 					/* see if can fold */
 					yystype *prev_string = &state->tokens.data[state->tokens.cnt-2];
 					if(prev_string->type == TOK_StringLiteral) {
