@@ -1,6 +1,7 @@
 /*
  *  Match a string against a list of patterns/regexes.
  *
+ *  Copyright (C) 2015 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2008 Sourcefire, Inc.
  *
  *  Authors: Török Edvin
@@ -584,11 +585,11 @@ static int add_newsuffix(struct regex_matcher *matcher, struct regex_list *info,
 	new->mindist = 0;
 	new->maxdist = 0;
 	new->offset_min = CLI_OFF_ANY;
-	new->length = len;
+	new->length[0] = len;
 
 	new->ch[0] = new->ch[1] |= CLI_MATCH_IGNORE;
-	if(new->length > root->maxpatlen)
-		root->maxpatlen = new->length;
+	if(new->length[0] > root->maxpatlen)
+		root->maxpatlen = new->length[0];
 
 	new->pattern = mpool_malloc(matcher->mempool, sizeof(new->pattern[0])*len);
 	if(!new->pattern) {
