@@ -33,10 +33,6 @@
 #include <dirent.h>
 #include <ctype.h>
 
-#include <openssl/ssl.h>
-#include <openssl/err.h>
-#include "libclamav/crypto.h"
-
 #include "libclamav/clamav.h"
 #include "libclamav/vba_extract.h"
 #include "libclamav/cltypes.h"
@@ -62,22 +58,22 @@ cli_ctx *convenience_ctx(int fd) {
     cli_ctx *ctx;
     struct cl_engine *engine;
 
-    ctx = malloc(sizeof(*ctx));
+    ctx = cli_calloc(1, sizeof(cli_ctx));
     if(!ctx){
-	printf("ctx malloc failed\n");
+	printf("ctx allocation failed\n");
         return NULL;
     }
 
     ctx->engine = engine = cl_engine_new();
     if(!(ctx->engine)){	    
-	printf("engine malloc failed\n");
+	printf("engine initialization failed\n");
         free(ctx);
 	return NULL;
     }	
 
-    ctx->fmap = cli_malloc(sizeof(struct F_MAP *));
+    ctx->fmap = cli_calloc(1, sizeof(struct F_MAP *));
     if(!(ctx->fmap)){
-	printf("fmap malloc failed\n");
+	printf("fmap initialization failed\n");
         free(engine);
         free(ctx);
 	return NULL;
@@ -1112,7 +1108,7 @@ static int sigtool_scandir (const char *dirname, int hex_output)
 
 int sigtool_vba_scandir (const char *dirname, int hex_output, struct uniq *U)
 {
-    int ret = CL_CLEAN, i, j, fd, data_len;
+    int ret = CL_CLEAN, i, fd, data_len;
     vba_project_t *vba_project;
     DIR *dd;
     struct dirent *dent;
@@ -1120,6 +1116,7 @@ int sigtool_vba_scandir (const char *dirname, int hex_output, struct uniq *U)
     char *fullname, vbaname[1024], *hash;
     unsigned char *data;
     uint32_t hashcnt;
+    unsigned int j;
 
     hashcnt = uniq_get(U, "_vba_project", 12, NULL);
     while(hashcnt--) {
